Type: Package
Package: BatchQC
Title: Batch Effects Quality Control Software
Version: 2.9.0
Date: 2026-02-10
Authors@R: c(
    person("Jessica", "Anderson", , "anderson.jessica@rutgers.edu", 
    role = c("aut"), comment = c(ORCID = "0000-0002-0542-9872")),
    person("W. Evan", "Johnson", , "wj183@njms.rutgers.edu",
    role = c("aut", "fnd"), comment = c(ORCID = "0000-0002-6247-6595")),
    person("Yaoan", "Leng", , "leng@bu.edu", role = c("ctb", "cre"),
    comment = c(ORCID = "0009-0002-3957-5250")),
    person("Solaiappan", "Manimaran", , "manimaran_1975@hotmail.com", role = "aut"),
    person("Heather", "Selby", , "selbyh@bu.edu", role = "ctb"),
    person("Claire", "Ruberman", , "claireruberman@gmail.com", role = "ctb"),
    person("Kwame", "Okrah", , "kwame.okrah@gmail.com", role = "ctb"),
    person("Hector Corrada", "Bravo", , "hcorrada@gmail.com", role = "ctb"),
    person("Michael", "Silverstein", , "msilver4@bu.edu", role = "ctb"),
    person("Regan", "Conrad", , "rdconrad@bu.edu", role = "ctb"),
    person("Zhaorong", "Li", , "zhaorong@bu.edu", role = "ctb"),
    person("Evan", "Holmes", , "evanjh19@gmail.com", role = "ctb"),
    person("Solomon", "Joseph", , "sj1136@njms.rutgers.edu", role = "ctb"),
    person("Howard", "Fan", , "hf268@njms.rutgers.edu", role = "ctb"),
    person("Sean", "Lu", , "sl1729@njms.rutgers.edu", role = "ctb")
  )
Description: Sequencing and microarray samples often are collected or
    processed in multiple batches or at different times. This often
    produces technical biases that can lead to incorrect results in the
    downstream analysis. BatchQC is a software tool that streamlines batch
    preprocessing and evaluation by providing interactive diagnostics,
    visualizations, and statistical analyses to explore the extent to
    which batch variation impacts the data. BatchQC diagnostics help
    determine whether batch adjustment needs to be done, and how
    correction should be applied before proceeding with a downstream
    analysis. Moreover, BatchQC interactively applies multiple common
    batch effect approaches to the data and the user can quickly see the
    benefits of each method. BatchQC is developed as a Shiny App. The
    output is organized into multiple tabs and each tab features an
    important part of the batch effect analysis and visualization of the
    data. The BatchQC interface has the following analysis groups:
    Summary, Differential Expression, Median Correlations, Heatmaps,
    Circular Dendrogram, PCA Analysis, Shape, ComBat and SVA.
License: MIT + file LICENSE
URL: https://github.com/wejlab/BatchQC
BugReports: https://github.com/wejlab/BatchQC/issues
Depends: R (>= 4.5.0)
Imports: data.table, DESeq2, dplyr, EBSeq, edgeR, FNN, ggdendro,
        ggnewscale, ggplot2, ggpubr, Harman, limma, matrixStats,
        methods, MASS, pheatmap, RColorBrewer, reader, reshape2, scran,
        shiny, shinyjs, shinythemes, stats, SummarizedExperiment, sva,
        S4Vectors, tibble, tidyr, tidyverse, umap, utils
Suggests: BiocManager, BiocStyle, bladderbatch, curatedTBData,
        devtools, knitr, lintr, MultiAssayExperiment, plotly,
        rmarkdown, spelling, testthat (>= 3.0.0)
VignetteBuilder: knitr
biocViews: BatchEffect, GeneExpression, GraphAndNetwork, Microarray,
        Normalization, PrincipalComponent, Sequencing, Software,
        Visualization, QualityControl, RNASeq, Preprocessing,
        DifferentialExpression, ImmunoOncology
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Collate: 'Normalization.R' 'batch_correction.R' 'color_palette.R'
        'compute_aic.R' 'covariates_not_confounded.R' 'data.R'
        'dendrogram.R' 'dendrogram_alpha_numeric_check.R'
        'dendrogram_color_palette.R' 'differential_expression.R'
        'explained_variation.R' 'global_vars.R' 'heatmap.R'
        'heatmap_num_to_char_convertor.R' 'import.R' 'kBET-utils.R'
        'kBET.R' 'lambda_statistic.R' 'negative_binomial_check.R'
        'pca.R' 'preprocess.R' 'runApp.R' 'summary_statistics.R'
        'umap.R' 'variation_ratios.R' 'volcano_plot.R'
git_url: https://git.bioconductor.org/packages/BatchQC
git_branch: devel
git_last_commit: c30deb7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:29:50 UTC; biocbuild
Author: Jessica Anderson [aut] (ORCID: <https://orcid.org/0000-0002-0542-9872>),
  W. Evan Johnson [aut, fnd] (ORCID:
    <https://orcid.org/0000-0002-6247-6595>),
  Yaoan Leng [ctb, cre] (ORCID: <https://orcid.org/0009-0002-3957-5250>),
  Solaiappan Manimaran [aut],
  Heather Selby [ctb],
  Claire Ruberman [ctb],
  Kwame Okrah [ctb],
  Hector Corrada Bravo [ctb],
  Michael Silverstein [ctb],
  Regan Conrad [ctb],
  Zhaorong Li [ctb],
  Evan Holmes [ctb],
  Solomon Joseph [ctb],
  Howard Fan [ctb],
  Sean Lu [ctb]
Maintainer: Yaoan Leng <leng@bu.edu>
