# Generated by roxygen2: do not edit by hand

export(BatchQC)
export(DE_analyze)
export(EV_plotter)
export(EV_table)
export(PCA_plotter)
export(batch_correct)
export(batch_design)
export(batchqc_explained_variation)
export(bisect)
export(bladder_data_upload)
export(color_palette)
export(compute_aic)
export(compute_lambda)
export(confound_metrics)
export(cor_props)
export(covariates_not_confounded)
export(cramers_v)
export(dendrogram_alpha_numeric_check)
export(dendrogram_color_palette)
export(dendrogram_plotter)
export(goodness_of_fit_nb)
export(heatmap_num_to_char_converter)
export(heatmap_plotter)
export(is_design_balanced)
export(kBET)
export(normalize_SE)
export(plot_kBET)
export(process_dendrogram)
export(pval_plotter)
export(pval_summary)
export(ratio_plotter)
export(run_kBET)
export(run_lambda)
export(std_pearson_corr_coef)
export(summarized_experiment)
export(summary_stats_EV_table)
export(tb_data_upload)
export(umap)
export(variation_ratios)
export(volcano_plot)
import(DESeq2)
import(EBSeq)
import(Harman)
import(RColorBrewer)
import(SummarizedExperiment)
import(dplyr)
import(edgeR)
import(ggdendro)
import(ggnewscale)
import(ggplot2)
import(ggpubr)
import(pheatmap)
import(reader)
import(reshape2)
import(scran)
import(shinyjs, except=show)
import(shinythemes)
import(sva)
import(tibble)
import(tidyr)
import(tidyverse)
import(umap)
import(utils)
importFrom(FNN,get.knn)
importFrom(MASS,glm.nb)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,DataFrame)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,rbindlist)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,removeBatchEffect)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(methods,is)
importFrom(stats,AIC)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,dist)
importFrom(stats,dmultinom)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.control)
importFrom(stats,kruskal.test)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnbinom)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(tidyr,pivot_longer)
importFrom(utils,data)
